/*
    Dirichlet Process Mixture of Hidden Markov Models: Accompanying Software
    Copyright (C) <2009>  <Kristin P. Lennox and David B. Dahl>

    Contact: lennox@stat.tamu.edu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

package HMMDP.BaseDistributions;

//Originally in DBD turnip.pepclust package
// Modified to use KPL Vector class

import HMMDP.*;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;

public class BivariateNormal implements Distribution<Vector> {

  public final Vector mean;
  public final Scale scale;

  public BivariateNormal(Scale scale) {
    this(new Vector(0,0),scale);
  }

  public BivariateNormal(Vector mean, Scale scale) {
    this.mean = mean;
    this.scale = scale;
  }

    @Override
  public Object clone() {
    Object result = null;
    try {
      result = super.clone();
    } catch (CloneNotSupportedException e) {}
    return result;
  }

  public double logDensity(Vector x) {
    return logNormalizingConstant() + logDensityKernel(x);
  }

  public double logDensityKernel(Vector x) {
        try {
            double z1 = (x.phi() - mean.phi()) / scale.std1;
            double z2 = (x.psi() - mean.psi()) / scale.std2;
            return -1.0 / (2.0 * (1.0 - scale.corr * scale.corr)) * (z1 * z1 - 2 * scale.corr * z1 * z2 + z2 * z2);
        } catch (Exception ex) {
            Logger.getLogger(BivariateNormal.class.getName()).log(Level.SEVERE, null, ex);
            return Math.log(0.0);
        }
  }

  public double logNormalizingConstant() {
    return -Math.log(scale.std1*scale.std2*Math.sqrt(1.0-scale.corr*scale.corr)) - LOG_2_PI;
  }
  
  public Vector sample(Random random) {
        try {
            double z1 = random.nextGaussian();
            double z2 = random.nextGaussian();
            double x1 = mean.phi() + scale.std1 * z1;
            double x2 = mean.psi() + scale.std2 * (scale.corr * z1 + Math.sqrt(1 - scale.corr * scale.corr) * z2);
            return new Vector(x1, x2);
        } catch (Exception ex) {
            // In case of failure, return empty vector.
            Logger.getLogger(BivariateNormal.class.getName()).log(Level.SEVERE, null, ex);
            return new Vector();
        }
  }

  private final static double LOG_2_PI = Math.log(2*Math.PI);



}
