/*
    Dirichlet Process Mixture of Hidden Markov Models: Accompanying Software
    Copyright (C) <2009>  <Kristin P. Lennox and David B. Dahl>

    Contact: lennox@stat.tamu.edu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package HMMDP.BaseDistributions;

import HMMDP.*;
import java.util.Random;

// Original code from turnip.pepclust package.
// Modified to make phi_, psi_ private, to allow for empty vectors
// Note that Vector does not restrict values to fall between -pi and pi
public class Vector {

  private double phi_;
  private double psi_;
  public final boolean isEmpty;
  
  public Vector(){
      phi_=0;
      psi_=0;
      isEmpty=true;
  }
  
  public Vector(double phi, double psi) {
    phi_ = phi;
    psi_ = psi;
    isEmpty=false;
  }

  // Create an Angle from a Vector
  public Angles convertToAngle(){
      return new Angles(phi_,psi_);
  }
  
  public static Vector sample(double minA, double maxA, double minB, double maxB, Random random) {
    return new Vector((maxA-minA)*(random.nextDouble()-0.5)+(maxA+minA)/2.0, (maxB-minB)*(random.nextDouble()-0.5)+(maxB+minB)/2.0);
  }

  public String toString() {
    if(!isEmpty)
      return phi_ + " " + psi_;
    else
      return "NA";
  }

  // Accessor functions
  public double phi() throws Exception {
      if(!isEmpty)
          return phi_;
      else
          throw new Exception("Empty Vector.");
  }
  
    public double psi() throws Exception {
      if(!isEmpty)
          return psi_;
      else
          throw new Exception("Empty Vector.");
  }

}
