/*
    Dirichlet Process Mixture of Hidden Markov Models: Accompanying Software
    Copyright (C) <2009>  <Kristin P. Lennox and David B. Dahl>

    Contact: lennox@stat.tamu.edu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

package HMMDP;

// A parameter set for an HMM model. Does not allow empty angles.

import HMMDP.BaseDistributions.Angles;
import HMMDP.BaseDistributions.Scale;

/**
 * Contains an Angles object and a Scale object, which serve as the parameters
 * of a BivariateVonMises distribution.
 */
public class ParameterSet {
    public final Angles center;
    public final Scale scale;

    public ParameterSet(Angles center, Scale scale) throws Exception {
  
        if(center.isEmpty) throw new Exception("Empty Angles objects not allowed.");
 
        this.center=center;
        this.scale=scale;
    }
    
    public String toString(){
        return center.toString()+" "+scale.toString();
    }
}
